/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * -----------------------------------------------------------------------------------------
 *  Comedi acquistion block: Computational function for the Comedi Analog Data Write block
 *
 *  Revision 1.1   2009/12/17  12:08:00 elaine
 *  Initial revision
 * 
 *  Revision 1.2   2009/12/24  16:40:00 elaine
 *  Improved pointer allocation. Included error checking. 
 *
 *  ----------------------------------------------------------------------------------------
 */


/* required header files */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h> 
#include <fcntl.h> 
#include <ctype.h>
#include <sys/types.h> 
#include <asm/types.h> 
#include "../includes/scicos_block4.h"  /* Specific for scicos block developement */
#include "../includes/machine.h"
#include <math.h>
#include <comedilib.h>     /* Specific for Comedi Interface */
#include <sys/socket.h>    /* Specific for socket connection */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "../includes/comedisrv.h"


/* simulation flag constants */
#define Init           4   /* Initialization */ 
#define OutputUpdate   1
#define StateUpdate    2
#define Integration    0
#define Mode           9   /* Zero Crossing */
#define EventScheduler 3
#define Ending         5   



/* simulation function for the Acquisition of analog signals */
void comedi_net_an_out(scicos_block *block, int flag) {


	static int convflag = 1;
	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	int execStatus;
	static char *openStat, *openMsg, *recvOpen;
	static char *dataWriteStat, *dataWriteMsg, *recvDataWrite;
	static char *maxDataStat, *maxDataMsg, *recvMaxData;
	static char *rangeStat, *rangeMsg, *recvRange;
	static char *fromPhysStat, *fromPhysMsg, *recvFromPhys;


	/* customer inputs from Scicos block */
	char ip[BUFFERSIZE];
	char dev[BUFFERSIZE];
	char subdev[BUFFERSIZE];
	char channel[BUFFERSIZE];
	char range[BUFFERSIZE];
	char aref[BUFFERSIZE];

	/* Scicos input and output vectors */
	int *ipar = block->ipar;
	double *inptr = block->inptr[0];

	/* customer input sizes */
	int devSize, ipSize,subdevSize,channSize,rangeSize,arefSize;

	/* Scicos custom function to get strings from blocks to C programs */
	extern int cvstr_();


	lsampl_t data = 0;
	double dataPhys = 0.000;
	char dataPhysChar[BUFFERSIZE];

	
	switch(flag) {  

		case Init: // Initializes variables, starts memory allocation  


			/* initializes input parameter sizes coming from Scicos */ 
			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];
			rangeSize = ipar[4];
			arefSize = ipar[5];
			
			
			/* initializes input parameters coming from Scicos */ 
			cvstr_(&ipSize,&ipar[6],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[6+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[6+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[6+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';

			cvstr_(&rangeSize, &ipar[6+devSize+ipSize+subdevSize+channSize], range, &convflag, rangeSize*sizeof(char));
			range[rangeSize] = '\0';
			
			cvstr_(&arefSize, &ipar[6+devSize+ipSize+subdevSize+channSize+rangeSize], aref, &convflag, arefSize*sizeof(char));
			aref[arefSize] = '\0';
			
			/* memory allocation to character pointers */

			/* comedi_open */
			if ((openMsg  = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}
			if ((openStat = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;		
			}
			if ((recvOpen = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}

			/* comedi_get_maxdata */
			if ((maxDataStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((maxDataMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvMaxData = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}

			/* comedi_get_range */
			if ((rangeStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}
			if ((rangeMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}
			if ((recvRange = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}

			/* comedi_data_write */
			if ((dataWriteStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((dataWriteMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}
			if ((recvDataWrite = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}

			/* comedi_from_phys */
			if ((fromPhysStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((fromPhysMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvFromPhys = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;			
			}

			/* acquisition functions */
			openComediDevice(openStat, openMsg, recvOpen, ip, dev);
			getMaxData(maxDataStat, maxDataMsg, recvMaxData, ip, dev, subdev, channel);
			getRange(rangeStat, rangeMsg, recvRange, ip, dev, subdev, channel, range);
                           
			/*
			 * sets value to initialize channel
			 */
			 
			/* establishes a connection with the server. If not possible, exits */
			if ((sockfd = connectToServer(ip)) > 0){
		

				/* assembles message to send to server */
				dataPhys = 0.000;
				sprintf(dataPhysChar, "%f", dataPhys);
				
				remoteComediFromPhys(dataPhysChar,recvMaxData, fromPhysMsg);
				
				

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,fromPhysMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_from_phys executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,fromPhysStat)) > 0){

						/* checks the status of the comedi_from_phys function. If status ok 
						 * the program continues. If not ok program aborts */

						
						rc = checkCmdStatus(fromPhysStat, recvFromPhys);
				                  if (rc == 0){
				                  
						     
												      
						    }
					      else{
						      Scierror(888,"Error in comedi_from_phys.\n");
				                        set_block_error(-1);
						      }

					      }
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				      }

				      }

			      else{
				      Scierror(888,"It was not possible to send message: '%s' to server\n", fromPhysMsg);
				               set_block_error(-1);
			      }

			      close(sockfd);

							

			}
			else{
				 Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
			}
			
			
                           /*
			 * writes value to initialize channel
			 */
			 
			/* establishes a connection with the server. If not possible, exits */
			if ((sockfd = connectToServer(ip)) > 0) {
		

				/* assembles message to send to server */
				remoteComediDataWrite(dev,subdev, channel,range, aref,recvFromPhys, dataWriteMsg);
				

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,dataWriteMsg)) >=  0){
				
				

					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_data_write executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,dataWriteStat)) > 0){

					

						/* checks the status of the comedi_data_write function. If status ok 
						 * the program continues. If not ok program aborts 
						 */

						
						rc = checkCmdStatus(dataWriteStat, recvDataWrite);
						
				                  if (rc == 0){
				                  
						     // printf("VALOR ESCRITO: '%s'\n",recvDataWrite);
						      					      
						    }
					      else{
						      Scierror(888,"Error in comedi_data_write.\n");
				                        set_block_error(-1);
						      }

					      }
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
					    
				      }

				      }

			      else{
				      Scierror(888,"It was not possible to send message: '%s' to server\n", dataWriteMsg);
				               set_block_error(-1);
			      }

			      close(sockfd);

							

			}
			else{
				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
			}
			
			break;

  case OutputUpdate:

		      
		      /* assignment from Scicos block inputs to program variables */
		     	
				/* initializes input parameter sizes coming from Scicos */ 
			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];
			rangeSize = ipar[4];
			arefSize = ipar[5];
			
			
			/* initializes input parameters coming from Scicos */ 
			cvstr_(&ipSize,&ipar[6],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[6+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[6+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[6+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';

			cvstr_(&rangeSize, &ipar[6+devSize+ipSize+subdevSize+channSize], range, &convflag, rangeSize*sizeof(char));
			range[rangeSize] = '\0';
			
			cvstr_(&arefSize, &ipar[6+devSize+ipSize+subdevSize+channSize+rangeSize], aref, &convflag, arefSize*sizeof(char));
			aref[arefSize] = '\0';
		
		  
		      sockfd = -1;

	
    
		      
		         /*
			 * sets value to initialize channel
			 */
			 
			/* establishes a connection with the server. If not possible, exits. Also checks if the last write was ok */
			if ((sockfd = connectToServer(ip)) > 0) {
		

				/* assembles message to send to server */
				dataPhys = inptr[0]; /* gets the user input */
				sprintf(dataPhysChar, "%f", dataPhys);
				//printf("recvMaxData: '%s'\n", recvMaxData);

				remoteComediFromPhys(dataPhysChar,recvMaxData, fromPhysMsg);
				
				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,fromPhysMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_from_phys executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,fromPhysStat)) > 0){

						/* checks the status of the comedi_from_phys function. If status ok 
						 * the program continues. If not ok program aborts */

						//printf("client:valor do socket %d\n",sockfd);
						rc = checkCmdStatus(fromPhysStat, recvFromPhys);
				                  if (rc == 0){
				                  
						     // printf("VALOR QUE SERA ESCRITO: '%s'\n",recvFromPhys);
						     						      
						    }
					      else{
						     Scierror(888,"Error in comedi_from_phys.\n");
				               set_block_error(-1);
						      }

					      }
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				      }

				      }

			      else{
				      Scierror(888,"It was not possible to send message: '%s' to server\n", fromPhysMsg);
				               set_block_error(-1);
			      }

			      close(sockfd);

							

			}
			else{
				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				end_scicos_sim();
			}
			
			
                           /*
			 * writes value to initialize channel
			 */
			 
			/* establishes a connection with the server. If not possible, exits */
			if (((sockfd = connectToServer(ip)) > 0)) {
		

				/* assembles message to send to server */
				remoteComediDataWrite(dev,subdev, channel,range, aref,recvFromPhys, dataWriteMsg);

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,dataWriteMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_data_write executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,dataWriteStat)) > 0){

						/* checks the status of the comedi_data_write function. If status ok 
						 * the program continues. If not ok program aborts 
						 */

						
						rc = checkCmdStatus(dataWriteStat, recvDataWrite);
				                  if (rc == 0){
				                  
						      //printf("VALOR ESCRITO: '%s'\n",recvDataWrite);
						     
						    }
					      else{
						      Scierror(888,"Error in comedi_data_write.\n");
				               set_block_error(-1);
						      }

					      }
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				      }

				      }

			      else{
				       Scierror(888,"It was not possible to send message: '%s' to server\n", fromPhysMsg);
				               set_block_error(-1);
			      }

			      close(sockfd);

							

			}
			else{
				 Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				end_scicos_sim();
			}
			



		      break;

		      
  
  case Ending:
  
  
  
  /*
			 * sets value to initialize channel
			 */
			 

		

			/* establishes a connection with the server. If not possible, exits */
			if (((sockfd = connectToServer(ip)) > 0)) {
		

				/* assembles message to send to server */
				dataPhys = 0.000;
				sprintf(dataPhysChar, "%f", dataPhys);
				//printf("recvMaxData: '%s'\n", recvMaxData);

				remoteComediFromPhys(dataPhysChar,recvMaxData, fromPhysMsg);
				


				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,fromPhysMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_from_phys executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,fromPhysStat)) > 0){

						/* checks the status of the comedi_from_phys function. If status ok 
						 * the program continues. If not ok program aborts */

						//printf("client:valor do socket %d\n",sockfd);
						rc = checkCmdStatus(fromPhysStat, recvFromPhys);
				                  if (rc == 0){
				                  
						     // printf("VALOR QUE SERA ESCRITO: '%s'\n",recvFromPhys);
						     						      
						    }
					      else{
						     Scierror(888,"Error in comedi_from_phys.\n");
				               set_block_error(-1);
						      }

					      }
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				      }

				      }

			      else{
				      Scierror(888,"It was not possible to send message: '%s' to server\n", fromPhysMsg);
				               set_block_error(-1);
			      }

			      close(sockfd);

							

			}
			else{
				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				//end_scicos_sim();
			}

			
			
                           /*
			 * writes value to initialize channel
			 */
			 
			/* establishes a connection with the server. If not possible, exits */
			if (((sockfd = connectToServer(ip)) > 0)) {
		

				/* assembles message to send to server */
				remoteComediDataWrite(dev,subdev, channel,range,aref, recvFromPhys, dataWriteMsg);

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits 
				 */
				if((sentBytes = clientSender(sockfd,dataWriteMsg)) >=  0){
					
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_data_write executed remotely and the 
					 * msg returned. If not, exits. 
					 */
					if ((receiveStatus = clientReceiver(sockfd,dataWriteStat)) > 0){

						/* checks the status of the comedi_data_write function. If status ok 
						 * the program continues. If not ok program aborts 
						 */

						//printf("dataWriteStat %s\n",dataWriteStat);
						rc = checkCmdStatus(dataWriteStat, recvDataWrite);
				                  if (rc == 0){
				                  
						     // printf("VALOR ESCRITO: '%s'\n",recvDataWrite);
						     
						    }
					      else{
						      Scierror(888,"Error in comedi_data_write.\n");
				               set_block_error(-1);
						      }

					      }
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				      }

				      }

			      else{
				       Scierror(888,"It was not possible to send message: '%s' to server\n", fromPhysMsg);
				               set_block_error(-1);
			      }

			      close(sockfd);

							

			}
			else{
				 Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				//end_scicos_sim();
			}
  
		


		      /* frees all allocated pointers */
		      free(openMsg);
		      openMsg = NULL;

		      free(openStat);
		      openStat = NULL;

		      free(recvOpen);
		      recvOpen = NULL;

		      free(dataWriteStat);
		      dataWriteStat = NULL;

		      free(dataWriteMsg);
		      dataWriteMsg = NULL;

		      free(recvDataWrite);
		      recvDataWrite = NULL;
		      
		      free(maxDataStat);
		      maxDataStat = NULL;
		      
		      free(maxDataMsg);
		      maxDataMsg = NULL;
		      
		      free(recvMaxData);
		      recvMaxData = NULL;
		      
		      free(rangeStat);
		      rangeStat = NULL;
		      
		      free(rangeMsg);
		      rangeMsg = NULL;
		      
		      free(recvRange);
		      recvRange = NULL;
		      
		      free(fromPhysStat);
		      fromPhysStat = NULL;
		      
		      free(fromPhysMsg);
		      fromPhysMsg = NULL;
		      
		      free(recvFromPhys);
		      recvFromPhys = NULL;
		      
		      
		      
      break;	  
        
  } // close the switch
   
} // close the function 

