/*
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de Mattos Silva
 * Projeto de Graduacao - Adequacao do Software
 * de Simulacao Scicos para Aquisicao de Dados
 * e o Controle em Tempo Real
 * -----------------------------------------------------------------------------------------
 *  Comedi acquistion block: Computational function for the Comedi Analog Data Write block
 *
 *  Revision 1.1   2009/12/17  12:08:00 elaine
 *  Initial revision
 *
 *  Revision 1.2   2009/12/25  10:49:00 elaine
 *  Improved pointer allocation. Included error checking. 
 *  ----------------------------------------------------------------------------------------
 */


/* required header files */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h> 
#include <fcntl.h> 
#include <ctype.h>
#include <sys/types.h> 
#include <asm/types.h> 
#include "../includes/scicos_block4.h"  /* Specific for scicos block developement */
#include "../includes/machine.h"
#include <math.h>
#include <comedilib.h>     /* Specific for Comedi Interface */
#include <sys/socket.h>    /* Specific for socket connection */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "../includes/comedisrv.h"


/* simulation flag constants */
#define Init           4   /* Initialization */ 
#define OutputUpdate   1
#define StateUpdate    2
#define Integration    0
#define Mode           9   /* Zero Crossing */
#define EventScheduler 3
#define Ending         5   



/* simulation function for the Acquisition of analog signals */
void comedi_net_dg_out(scicos_block *block, int flag) {


	static int convflag = 1;
	int sockfd = -1 , sentBytes = 0, rc = 1, receiveStatus = 0;
	int execStatus;
	static char *openStat, *openMsg, *recvOpen;
	static char *dioWStat, *dioWMsg, *recvDioW;

	/* customer inputs from Scicos block */
	char ip[BUFFERSIZE];
	char dev[BUFFERSIZE];
	char subdev[BUFFERSIZE];
	char channel[BUFFERSIZE];

	/* Scicos input and output vectors */
	int *ipar = block->ipar;
	double *inptr = block->inptr[0];
	double *rpar = block->rpar;

	/* customer input sizes */
	int devSize, ipSize,subdevSize,channSize,rangeSize;

	/* Scicos custom function to get strings from blocks to C programs */
	extern int cvstr_();

	lsampl_t data = 0;
	double dataPhys = 0.000;
	double threshold;
	char dataChar[BUFFERSIZE];


	switch(flag) {  

		case Init: // Initializes variables, starts memory allocation  

			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];
			threshold = rpar[0];

			cvstr_(&ipSize,&ipar[4],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			

			cvstr_(&devSize,&ipar[4+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';

			cvstr_(&subdevSize, &ipar[4+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[4+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';

			/* memory allocation to character pointers */

			//printf("Inicio do grupo de mallocs: comedi_net_dg_out.c\n");

			/* comedi_open */
			if ((openMsg  = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((openStat = (char *)scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvOpen = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}

			/* comedi_dio_write */
			if ((dioWStat = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((dioWMsg = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}
			if ((recvDioW = (char *) scicos_malloc((unsigned)(BUFFERSIZE * sizeof(char)))) == NULL){
				set_block_error(-16);
				return;
			}

					
			/*
			 * opens device 
			 */
			 
			/* establishes a connection with the server. If not possible, exits */
			if ((sockfd = connectToServer(ip)) > 0){

				/* assembles message to send to server */
				remoteComediOpen(dev,openMsg);

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits */
				if((sentBytes = clientSender(sockfd,openMsg)) >=  0){

					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_open executed remotely and the 
					 * msg returned. If not, exits. */
					if ((receiveStatus = clientReceiver(sockfd,openStat)) > 0){

						/* checks the status of the comedi_open function. If status ok 
						 * the program continues. If not ok program aborts */

						//printf("client:valor do socket %d\n",sockfd);
						rc = checkCmdStatus(openStat, recvOpen);
						if (rc != 0){
							//printf("Error on comedi_open.\n");
							Scierror(888,"Error on comedi_open.\n");
						  set_block_error(-1);
						
							}

						//printf("comedi_open message: '%s'\n",recvOpen);
						}

					else{
						Scierror(888,"It was not possible to receive message from server\n");
						set_block_error(-1);
						}

					}
				else{
					Scierror(888,"It was not possible to send message: '%s' to server\n",openMsg);
					set_block_error(-1);

					}

				close(sockfd);

				}
			else{

				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
			}


			/*
			 * initializes channel 
			 */
			 
			/* establishes a connection with the server. If not possible, exits */
			if ((sockfd = connectToServer(ip)) > 0) {

				/* assembles message to send to server */
				data = 0;
				sprintf(dataChar, "%d", data);
				remoteComediDioWrite(dev,subdev, channel, dataChar, dioWMsg);

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits */
				if((sentBytes = clientSender(sockfd,dioWMsg)) >=  0){

					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_dio_write executed remotely and the 
					 * msg returned. If not, exits. */

					if ((receiveStatus = clientReceiver(sockfd,dioWStat)) > 0){
						/* checks the status of the comedi_dio_write function. If status ok 
						 * the program continues. If not ok program aborts */
						 
						//printf("RETORNO DO DIO_WRITE: '%s'\n",dioWStat);
						rc = checkCmdStatus(dioWStat, recvDioW);
						if (rc == 0){
							//printf("DIO WRITE: '%s'\n",recvDioW);
							}

						else{
							Scierror(888,"Error in comedi_dio_write.\n");
							set_block_error(-1);
						}
						} //end if-receiveStatus
				      else{
					      Scierror(888,"It was not possible to receive msgs from the server\n");
				               set_block_error(-1);
				      }

				      } //end if-sentBytes

			      else{
				      Scierror(888,"It was not possible to send message: '%s' to server\n", dioWMsg);
				      set_block_error(-1);

				      }
			      close(sockfd);

			      } //end if-sockfd
			else{
				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
			}

			break;

		case OutputUpdate:

			/* assignment from Scicos block inputs to program variables */

			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];
			threshold = rpar[0];

			cvstr_(&ipSize,&ipar[4],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[4+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[4+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[4+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';

			sockfd = -1;


			/*
			 * sets value to initialize channel
			 */
			 
			
			/* establishes a connection with the server. If not possible, exits. Also checks if the last write was ok */
			if ((sockfd = connectToServer(ip)) > 0) {

				/* assembles message to send to server */
				dataPhys = inptr[0]; /* gets the user input */
								
				if (dataPhys > threshold)
				  data = 1;
				else
				  data = 0;
				  
				sprintf(dataChar, "%d", data);
					
				remoteComediDioWrite(dev,subdev, channel, dataChar, dioWMsg);

				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits */
				if((sentBytes = clientSender(sockfd,dioWMsg)) >=  0){

					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_dio_write executed remotely and the 
					 * msg returned. If not, exits. */
					if ((receiveStatus = clientReceiver(sockfd,dioWStat)) > 0){
						/* checks the status of the comedi_dio_write function. If status ok 
						 * the program continues. If not ok program aborts */

						rc = checkCmdStatus(dioWStat, recvDioW);
						if (rc == 0){
							//printf("DIO WRITE: '%s'\n",recvDioW);
							}

						else{
							Scierror(888,"Error in comedi_dio_write.\n");
							set_block_error(-1);
							}
						}
					else{
						Scierror(888,"It was not possible to receive msgs from the server\n");
						set_block_error(-1);
						}
					}
				else{
					Scierror(888,"It was not possible to send message: '%s' to server\n", dioWMsg);
					set_block_error(-1);
					}
				close(sockfd);
				}
			else{
				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				end_scicos_sim();
			}
			
			break;

		case Ending:

			/* assignment from Scicos block inputs to program variables */
		     	
			ipSize = ipar[0];
			devSize = ipar[1];
			subdevSize = ipar[2];
			channSize = ipar[3];
			threshold = rpar[0];


                        cvstr_(&ipSize,&ipar[4],ip,&convflag,ipSize*sizeof(char));
			ip[ipSize] = '\0';
			
			cvstr_(&devSize,&ipar[4+ipSize],dev,&convflag,devSize*sizeof(char));
			dev[devSize] = '\0';
	
			cvstr_(&subdevSize, &ipar[4+devSize+ipSize], subdev, &convflag, subdevSize*sizeof(char));
			subdev[subdevSize] = '\0';

			cvstr_(&channSize, &ipar[4+devSize+ipSize+subdevSize], channel, &convflag, channSize*sizeof(char));
			channel[channSize] = '\0';


			/* establishes a connection with the server. If not possible, exits */
			if ((sockfd = connectToServer(ip)) > 0) {
				
				/* assembles message to send to server */
				data = 0;
				sprintf(dataChar, "%d", data);
				remoteComediDioWrite(dev,subdev, channel, dataChar, dioWMsg);
							
				/* sends message to server. If the number of bytes is more than 0 receives 
				 * message from server. If not, exits */
				if((sentBytes = clientSender(sockfd,dioWMsg)) >=  0){

							
					/* receives msg from server. If the number of bytes read is more than 
					 * 0 checks the return code from comedi_dio_write executed remotely and the 
					 * msg returned. If not, exits. */
					if ((receiveStatus = clientReceiver(sockfd,dioWStat)) > 0){

						/* checks the status of the comedi_dio_write function. If status ok 
						 * the program continues. If not ok program aborts */

						rc = checkCmdStatus(dioWStat, recvDioW);
						if (rc == 0){
							//printf("DIO WRITE: '%s'\n",recvDioW);
							}
						else{
							Scierror(888,"Error in comedi_dio_write.\n");
							set_block_error(-1);						      
							}
					}

					else{
						Scierror(888,"It was not possible to receive msgs from the server\n");
						set_block_error(-1);
					}
				}
				else{
					Scierror(888,"It was not possible to send message: '%s' to server\n", dioWMsg);
					set_block_error(-1);
			      }

				close(sockfd);
				}
			else{
				Scierror(888,"It was not possible to open a connection with %s.\n",ip);
				set_block_error(-1);
				}


			/* frees all allocated pointers */
			//printf("Inicio do grupo de frees:comedi_net_dg_out.c\n");
			scicos_free(openMsg);
			openMsg = NULL;
			scicos_free(openStat);
			openStat = NULL;
			scicos_free(recvOpen);
			recvOpen = NULL;

			scicos_free(dioWStat);
			dioWStat = NULL;
			scicos_free(dioWMsg);
			dioWMsg = NULL;
			scicos_free(recvDioW);
			recvDioW = NULL;


			    
		      
      break;	  
        
  } // close the switch
   
} // close the function 

